@@ -1,7 +1,7 @@
 /* Inter thread serialization module for the Lua/APR binding.
  *
  * Author: Peter Odding <peter@peterodding.com>
- * Last Change: June 19, 2011
+ * Last Change: June 30, 2011
  * Homepage: http://peterodding.com/code/lua/apr/
  * License: MIT
  *
@@ -194,22 +194,13 @@ int push_tuple(lua_State *L, void *tuple)
       }
       case TV_OBJECT: {
         lua_apr_objtype *objtype;
-        lua_apr_refobj *object, *reference;
+        lua_apr_refobj *object;
         ptr += sizeof(packed_type);
         memcpy(&objtype, ptr, sizeof objtype);
         ptr += sizeof objtype;
         memcpy(&object, ptr, sizeof object);
         ptr += sizeof object;
-        reference = lua_newuserdata(L, sizeof(lua_apr_refobj));
-        if (reference == NULL) {
-          /* TODO Raise proper error message?
-           * TODO Cleanup references to already packed objects?! */
-          LUA_APR_DBG("#%i: Failed to unpack userdata!", i + 1);
-          return i;
-        }
-        reference->reference = object;
-        reference->unmanaged = 0;
-        init_object(L, objtype);
+        create_reference(L, objtype, object);
         break;
       }
       default:
